# !/usr/bin/python
import requests
import os
from html.parser import HTMLParser
from os.path import basename, join
import shutil

class MyHTMLParser(HTMLParser):
    def __init__(self, url, output_directory):
        HTMLParser.__init__(self)
        self.output_directory = output_directory
        self.url = url
        self.replacement_map = {}

    def handle_starttag(self, tag, attrs):
        attrs = dict(attrs)
        if tag == 'link':
            self.fetch_static_asset(attrs['href'])
        if tag == 'script' and 'src' in attrs:
            self.fetch_static_asset(attrs['src'])

            # print("Encountered a start tag:", tag)

    def handle_endtag(self, tag):
        return
        print("Encountered an end tag :", tag)

    def handle_data(self, data):
        return
        print("Encountered some data  :", data)

    def fetch_static_asset(self, url):
        print(self.output_directory, url)
        if url.split(':')[0] in ['http', 'https']:  # External resource
            print("WARNING: External resource", url, 'Skipping')
            return
        fetch_url = self.url + url
        output_filename = basename(fetch_url)
        resource_extension = url.split('.')[-1]
        resource_map = {
            'js': join('static', 'js'),
            'css': join('static', 'css'),
            'ico': None,
        }
        resource_type = resource_map.get(resource_extension, None)
        if resource_type is None:
            print("WARNING: Ignoring resource", url)
            return
        output_dir = join(self.output_directory, resource_map[resource_extension])
        if output_dir is None:  # We don't fetch icons
            return
        print("Fetching %s ->  %s/%s" % (fetch_url, output_dir, output_filename))
        os.makedirs(output_dir, mode=0o755, exist_ok=True)
        resource = requests.get(fetch_url)

        with open(join(output_dir, output_filename), 'w') as static_file:
            static_file.write(resource.text)
        self.replacement_map[url] = join(resource_extension, output_filename)


class TemplateMaker:
    def __init__(self, url, output_directory):
        self.url = url
        self.output_directory = output_directory

    def make(self):
        page = requests.get(self.url)
        parser = MyHTMLParser(self.url, self.output_directory)
        parser.feed(page.text)
        html = page.text

        # Replace resource urls with our uniformed static locations
        for old, new in parser.replacement_map.items():
            print(old, new)
            html = html.replace(old, new)

        # Create the index.html view
        views_dir = join(self.output_directory, 'views')
        os.makedirs(views_dir, mode=0o755, exist_ok=True)
        index_filename = join(views_dir, 'index.html')
        print("Creating ", index_filename)
        with open(index_filename, 'w') as index_file:
            index_file.write(html)

        # Add "app.py" required for some python app engines
        app_py = join(os.path.dirname((os.path.realpath(__file__))), 'app.py')
        print("%s -> %s" % (app_py, join(self.output_directory, 'app.py')))
        shutil.copy(app_py, join(self.output_directory, 'app.py'))
        print("Make completed.")

        # Add "requirements.txt"
        requirements = join(self.output_directory, 'requirements.txt')
        with open(requirements, 'w') as requirements_file:
            requirements_file.write('QuickWeb\n')
        print("-> %s" %  requirements)

