## -*- coding: utf-8 -*-
import imp
from os.path import join, basename, dirname
from quickweb import controller, template
from glob import glob
import cherrypy
import os

APP_NAME = None


# Set a default app root handler, to bind controllers until we find an "index" controller
class AppServerRoot(object):  # Application Server Root place holder

    def index(self):
        return template.render('index.html')
    index.exposed = True


Root = AppServerRoot()


def set_default_config():
    global Root
    # Enforce utf8 encoding
    cherrypy.config.update({'tools.encode.on': True
                               , 'tools.encode.encoding': 'utf-8'
                               , 'tools.encode.errors': 'replace'
                            })
    controller.app_root = Root

    cherrypy.tree.mount(controller.app_root, '/')


# Setup static file serve handling for 'static' sub-dirs
def set_static_dirs(application, static_dir):
    print("*** Checking for static dirs")
    for static_dir in glob(join(static_dir, '*')):
        print("- Setting static dir %s -> %s" % (basename(static_dir), static_dir))
        conf = {}
        conf['/' + basename(static_dir)] = {'tools.staticdir.dir': static_dir,
                                            'tools.staticdir.on': True
                                            }
        application.merge(conf)


# Load and setup controllers
def load_controllers(controller_dir):
    print("*** Checking for controllers")
    for controller_file in glob(join(controller_dir, '*.py')):
        module_name = APP_NAME + '.controllers.' + basename(controller_file)
        print("- Loading conroller ", module_name)
        imp.load_source(module_name, controller_file)


def set_views_dirs(template_dirs):
    print("*** Setting views  directory to ", template_dirs)
    template.set_directories(template_dirs)


app = cherrypy.Application(Root, script_name=None, config=None)


def setup_app(app_name, app_directory):
    print(app_directory)
    global APP_NAME
    APP_NAME = app_name
    set_default_config()
    print('=' * 10 + " Setting up " + app_name + " "+ '=' * 10)
    set_static_dirs(app, join(app_directory, 'static'))
    set_views_dirs(join(app_directory, 'views'))
    load_controllers(join(app_directory, 'controllers'))
    print('=' * 10 + ' Startup completed.')
